@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2011-2020, Intel Corporation
+* Copyright (c) 2011-2021, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -39,13 +39,19 @@
 
 #include "mhw_vdbox_mfx_interface.h"
 #include "mhw_vdbox_hcp_interface.h"
+#include "mhw_vdbox_avp_interface.h"
 #include "mhw_vdbox_huc_interface.h"
 #include "mhw_vdbox_vdenc_interface.h"
+#include "mhw_vdbox_hcp_itf.h"
 
 #include "media_interfaces_mhw.h"
 
 #include "gfxmacro.h"
 
+#ifdef IGFX_MHW_INTERFACES_NEXT_SUPPORT
+#include "codec_hw_next.h"
+#endif
+
 //------------------------------------------------------------------------------
 // Macros specific to MOS_CODEC_SUBCOMP_HW sub-comp
 //------------------------------------------------------------------------------
@@ -376,8 +382,11 @@ protected:
     MhwSfcInterface                 *m_sfcInterface = nullptr;        //!< Pointer to Mhw sfc interface
     MhwVdboxMfxInterface            *m_mfxInterface = nullptr;        //!< Pointer to Mhw mfx interface
     MhwVdboxHcpInterface            *m_hcpInterface = nullptr;        //!< Pointer to Mhw hcp interface
+    MhwVdboxAvpInterface            *m_avpInterface = nullptr;        //!< Pointer to Mhw avp interface
     MhwVdboxHucInterface            *m_hucInterface = nullptr;        //!< Pointer to Mhw huc interface
     MhwVdboxVdencInterface          *m_vdencInterface = nullptr;      //!< Pointer to Mhw vdenc interface
+    std::shared_ptr<mhw::vdbox::hcp::Itf>   m_hcpItf   = nullptr;
+    std::shared_ptr<mhw::vdbox::vdenc::Itf> m_vdencItf = nullptr;
 
     CODECHAL_SSEU_SETTING const         *m_ssEuTable = nullptr;       //!< Pointer to the default SSEU settings table
     uint16_t                            m_numMediaStates = CODECHAL_NUM_MEDIA_STATES;  //!< number of media states
@@ -456,6 +465,32 @@ public:
         MhwInterfaces     *mhwInterfaces,
         bool              disableScalability = false);
 
+#ifdef IGFX_MHW_INTERFACES_NEXT_SUPPORT
+    CodechalHwInterface(
+        PMOS_INTERFACE    osInterface,
+        CODECHAL_FUNCTION codecFunction,
+        MhwInterfacesNext *mhwInterfacesNext,
+        bool              disableScalability = false);
+
+    //!
+    //! \brief    Get avp interface
+    //! \details  Get avp interface in codechal hw interface next
+    //!
+    //! \return    pointer to new AVP interface
+    //!
+    inline std::shared_ptr<mhw::vdbox::avp::Itf> GetAvpInterfaceNext()
+    {
+        if (m_hwInterfaceNext)
+        {
+            return m_hwInterfaceNext->GetAvpInterfaceNext();
+        }
+        else
+        {
+            return nullptr;
+        }
+    }
+#endif
+
     //!
     //! \brief    Copy constructor
     //!
@@ -536,7 +571,17 @@ public:
             MOS_Delete(m_sfcInterface);
             m_sfcInterface = nullptr;
         }
+#ifdef IGFX_MHW_INTERFACES_NEXT_SUPPORT
+        if (m_hwInterfaceNext)
+        {
+            MOS_Delete(m_hwInterfaceNext);
+            m_hwInterfaceNext = nullptr;
+        }
+#endif
 
+    #if MHW_HWCMDPARSER_ENABLED
+        mhw::HwcmdParser::DestroyInstance();
+    #endif
     }
 
     //!
@@ -659,6 +704,18 @@ public:
         return m_hcpInterface;
     }
 
+    //!
+    //! \brief    Get avp interface
+    //! \details  Get avp interface in codechal hw interface
+    //!
+    //! \return   [out] MhwVdboxAvpInterface*
+    //!           Interface got.
+    //!
+    MhwVdboxAvpInterface *GetAvpInterface()
+    {
+        return m_avpInterface;
+    }
+
     //!
     //! \brief    Get huc interface
     //! \details  Get huc interface in codechal hw interface 
@@ -1285,10 +1342,11 @@ public:
     //!           MOS_STATUS_SUCCESS if success, else fail reason
     //!
     MOS_STATUS SendHwSemaphoreWaitCmd(
-        PMOS_RESOURCE                               semaMem,
-        uint32_t                                    semaData,
-        MHW_COMMON_MI_SEMAPHORE_COMPARE_OPERATION   opCode,
-        PMOS_COMMAND_BUFFER                         cmdBuffer);
+        PMOS_RESOURCE                             semaMem,
+        uint32_t                                  semaData,
+        MHW_COMMON_MI_SEMAPHORE_COMPARE_OPERATION opCode,
+        PMOS_COMMAND_BUFFER                       cmdBuffer,
+        uint32_t                                  semaMemOffset = 0);
 
     //!
     //! \brief    Send mi atomic dword cmd
@@ -1570,6 +1628,11 @@ public:
 
     virtual bool UsesRenderEngine(CODECHAL_FUNCTION codecFunction, uint32_t standard);
 
+    virtual bool Uses2PlanesInputSurfaceFilmGrain()
+    {
+        return false;
+    }
+
     //!
     //! \brief    Get film grain kernel info
     //! \details  Get kernel base and size
@@ -1587,11 +1650,59 @@ public:
         uint8_t*    &kernelBase,
         uint32_t    &kernelSize);
 
+    //!
+    //! \brief    Calculates the maximum size for AVP picture level commands
+    //! \details  Client facing function to calculate the maximum size for AVP picture level commands
+    //! \param    [in] mode
+    //!           Indicate the codec mode
+    //! \param    [out] commandsSize
+    //!           The maximum command buffer size
+    //! \param    [out] patchListSize
+    //!           The maximum command patch list size
+    //! \param    [in] params
+    //!           Indicate the command size parameters
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success, else fail reason
+    //!
+    virtual MOS_STATUS GetAvpStateCommandSize(
+            uint32_t                        mode,
+            uint32_t                        *commandsSize,
+            uint32_t                        *patchListSize,
+            PMHW_VDBOX_STATE_CMDSIZE_PARAMS params)
+    {
+        return MOS_STATUS_SUCCESS;
+    };
+
+    //!
+    //! \brief    Calculates maximum size for AVP tile level commands
+    //! \details  Client facing function to calculate maximum size for AVP tile level commands
+    //! \param    [in] mode
+    //!           Indicate the codec mode
+    //! \param    [out] commandsSize
+    //!            The maximum command buffer size
+    //! \param    [out] patchListSize
+    //!           The maximum command patch list size
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success, else fail reason
+    //!
+    virtual MOS_STATUS GetAvpPrimitiveCommandSize(
+            uint32_t                        mode,
+            uint32_t                        *commandsSize,
+            uint32_t                        *patchListSize)
+    {
+        return MOS_STATUS_SUCCESS;
+    };
+
+
     //! \brief    default disable vdbox balancing by UMD
     bool bEnableVdboxBalancingbyUMD = false;
     
     //! \brief    default disable the get vdbox node by UMD, decided by MHW and MOS
     bool m_getVdboxNodeByUMD = false;
+
+#ifdef IGFX_MHW_INTERFACES_NEXT_SUPPORT
+    CodechalHwInterfaceNext *m_hwInterfaceNext = nullptr;
+#endif
 };
 
 extern const MOS_SYNC_PARAMS                        g_cInitSyncParams;
