@@ -506,7 +506,7 @@ typedef union _MHW_MEMORY_OBJECT_CONTROL_PARAMS
         uint32_t                   : 1 ;
         uint32_t   Index           : 6 ;
         uint32_t                   : 25;
-    } Gen9;
+    } Gen9, Gen10, Gen11, Gen12, Gen12_5, Gen12_7;
     uint32_t       Value;
 } MHW_MEMORY_OBJECT_CONTROL_PARAMS, *PMHW_MEMORY_OBJECT_CONTROL_PARAMS;
 
@@ -590,6 +590,14 @@ enum WRITE_FLAG
     WRITE_WA  = 0x2,
 };
 
+
+typedef struct _MHW_MOCS_PARAMS
+{
+    uint32_t *mocsTableIndex;
+    uint8_t  bitFieldLow;
+    uint8_t  bitFieldHigh;
+} MHW_MOCS_PARAMS;
+
 typedef struct _MHW_RESOURCE_PARAMS
 {
     PMOS_RESOURCE                       presResource;
@@ -613,6 +621,7 @@ typedef struct _MHW_RESOURCE_PARAMS
     uint32_t                            shiftAmount;
     uint32_t                            shiftDirection;
     MOS_PATCH_TYPE                      patchType;
+    MHW_MOCS_PARAMS                     mocsParams;
 }MHW_RESOURCE_PARAMS, *PMHW_RESOURCE_PARAMS;
 
 typedef struct _MHW_GENERIC_PROLOG_PARAMS
@@ -625,6 +634,18 @@ typedef struct _MHW_GENERIC_PROLOG_PARAMS
     uint32_t                    dwStoreDataValue;
 } MHW_GENERIC_PROLOG_PARAMS, *PMHW_GENERIC_PROLOG_PARAMS;
 
+void inline InitMocsParams(
+    MHW_RESOURCE_PARAMS &hwResourceParam,
+    uint32_t            *addr,
+    uint8_t             bitFieldLow,
+    uint8_t             bitFieldHigh)
+{
+    hwResourceParam.mocsParams.mocsTableIndex = addr;
+    hwResourceParam.mocsParams.bitFieldLow    = bitFieldLow;
+    hwResourceParam.mocsParams.bitFieldHigh   = bitFieldHigh;
+    return;
+}
+
 MOS_STATUS Mhw_AddResourceToCmd_GfxAddress(
     PMOS_INTERFACE              pOsInterface,
     PMOS_COMMAND_BUFFER         pCmdBuffer,
@@ -676,7 +697,9 @@ MOS_STATUS Mhw_AllocateBb(
     PMHW_BATCH_BUFFER       pBatchBuffer,
     PMHW_BATCH_BUFFER       pBatchBufferList,
     uint32_t                dwSize,
-    uint32_t                batchCount=1);
+    uint32_t                batchCount = 1,
+    bool                    notLockable = false,
+    bool                    inSystemMem = false);
 
 MOS_STATUS Mhw_FreeBb(
     PMOS_INTERFACE          pOsInterface,
@@ -702,7 +725,7 @@ uint32_t Mhw_ConvertToTRMode(
 //*-----------------------------------------------------------------------------
 static __inline MOS_STATUS Mhw_AddCommandBB(
     PMHW_BATCH_BUFFER           pBatchBuffer,   // [in] Pointer to Batch Buffer
-    void                        *pCmd,           // [in] Command Pointer
+    const void                  *pCmd,          // [in] Command Pointer
     uint32_t                    dwCmdSize)      // [in] Size of command in bytes
 {
     uint8_t     *pbBatchPtr;
@@ -748,17 +771,22 @@ finish:
 static __inline MOS_STATUS Mhw_AddCommandCmdOrBB(
     void       *pCmdBuffer,     // [in] Pointer to Command Buffer
     void       *pBatchBuffer,   // [in] Pointer to Batch Buffer
-    void       *pCmd,           // [in] Command Pointer
-    uint32_t   dwCmdSize)      // [in] Size of command in bytes
+    const void *pCmd,           // [in] Command Pointer
+    uint32_t   dwCmdSize)       // [in] Size of command in bytes
 {
     if (pCmdBuffer)
     {
         return ((MOS_STATUS)Mos_AddCommand((PMOS_COMMAND_BUFFER)pCmdBuffer, pCmd, dwCmdSize));
     }
-    else
+    else if (pBatchBuffer)
     {
         return (Mhw_AddCommandBB((PMHW_BATCH_BUFFER)pBatchBuffer, pCmd, dwCmdSize));
     }
+    else
+    {
+        MHW_ASSERTMESSAGE("There is no valid command buffer or batch buffer.");
+        return MOS_STATUS_NULL_POINTER;
+    }
 }
 
 #endif // __MHW_UTILITIES_H__
