@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2017-2020, Intel Corporation
+* Copyright (c) 2017-2021, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -99,6 +99,9 @@ public:
 
         typename TVeboxCmds::VEBOX_SURFACE_STATE_CMD cmd1, cmd2;
 
+        MEDIA_FEATURE_TABLE    *pSkuTable = nullptr;
+        pSkuTable = m_osInterface->pfnGetSkuTable(m_osInterface);
+        MHW_CHK_NULL(pSkuTable);
         MHW_CHK_NULL(pCmdBuffer);
         MHW_CHK_NULL(pVeboxSurfaceStateCmdParams);
 
@@ -113,6 +116,13 @@ public:
             &cmd1,
             false,
             pVeboxSurfaceStateCmdParams->bDIEnable);
+
+        if (pVeboxSurfaceStateCmdParams->b3DlutEnable && MEDIA_IS_SKU(pSkuTable, FtrHeight8AlignVE3DLUTDualPipe))
+        {
+            cmd1.DW2.Height = MOS_ALIGN_CEIL((cmd1.DW2.Height + 1), 8) - 1;
+            MHW_NORMALMESSAGE("Align Input Height as 8x due to 3DlutEnable");
+        }
+
         Mos_AddCommand(pCmdBuffer, &cmd1, cmd1.byteSize);
         MHW_NORMALMESSAGE("Vebox input Height: %d, Width: %d;", cmd1.DW2.Height, cmd1.DW2.Width);
 
@@ -133,6 +143,12 @@ public:
                 cmd2.DW3.SurfaceFormat = cmd1.DW3.SurfaceFormat;
             }
 
+            if (pVeboxSurfaceStateCmdParams->b3DlutEnable && MEDIA_IS_SKU(pSkuTable, FtrHeight8AlignVE3DLUTDualPipe))
+            {
+                cmd2.DW2.Height = MOS_ALIGN_CEIL((cmd2.DW2.Height + 1), 8) - 1;
+                MHW_NORMALMESSAGE("Align Output Height as 8x due to 3DlutEnable");
+            }
+
             Mos_AddCommand(pCmdBuffer, &cmd2, cmd2.byteSize);
             MHW_NORMALMESSAGE("Vebox output Height: %d, Width: %d;", cmd2.DW2.Height, cmd2.DW2.Width);
         }
@@ -410,6 +426,28 @@ protected:
                 ((uint32_t)pCapPipeParams->FECSCParams.PostOffset[1]),
                 ((uint32_t)pCapPipeParams->FECSCParams.PostOffset[2]));
         }
+        else if (pVeboxIecpParams->bFeCSCEnable)
+        {
+            // Coeff is S2.16, so multiply the floating value by 65536
+            SET_COEFS(
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[0] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[1] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[2] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[3] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[4] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[5] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[6] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[7] * 65536)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscCoeff[8] * 65536)));
+            SET_INPUT_OFFSETS(
+                ((uint32_t)(pVeboxIecpParams->pfFeCscInOffset[0] * 128.0F)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscInOffset[1] * 128.0F)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscInOffset[2] * 128.0F)));
+            SET_OUTPUT_OFFSETS(
+                ((uint32_t)(pVeboxIecpParams->pfFeCscOutOffset[0] * 128.0F)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscOutOffset[1] * 128.0F)),
+                ((uint32_t)(pVeboxIecpParams->pfFeCscOutOffset[2] * 128.0F)));
+        }
         else if (pVeboxIecpParams->ColorSpace == MHW_CSpace_BT601)
         {
             SET_COEFS(16843, 33030, 6423, MOS_BITFIELD_VALUE((uint32_t)-9698, 19), MOS_BITFIELD_VALUE((uint32_t)-19070, 19), 28770, 28770, MOS_BITFIELD_VALUE((uint32_t)-24116, 19), MOS_BITFIELD_VALUE((uint32_t)-4652, 19));
