@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2011-2020, Intel Corporation
+* Copyright (c) 2011-2021, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -64,6 +64,12 @@
 #define NOISE_TEMPORALPIXELDIFF_THRESHOLD_DEFAULT       12
 #define NOISE_SUMABSTEMPORALDIFF_THRESHOLD_DEFAULT      128
 
+//!
+//! \brief Spatial Denoise Definitions
+//!
+#define NOSIE_GNE_CHROMA_THRESHOLD                  1850
+#define NOSIE_GNE_LUMA_THRESHOLD                    32000
+
 // Pixel Range Threshold Array Denoise Definitions for SKL+ 5x5 Bilateral Filter
 #define NOISE_BLF_RANGE_THRESHOLD_ADP_NLVL          1
 #define NOISE_BLF_RANGE_THRESHOLD_ADP_NLVL_MIN      65536
@@ -632,6 +638,7 @@ public:
                                             bTFF           = false;
                                             bTopField      = false;
                                             bBeCsc         = false;
+                                            bFeCsc         = false;
                                             bVeboxBypass   = false;
                                             b60fpsDi       = false;
                                             bQueryVariance = false;
@@ -658,6 +665,7 @@ public:
                                             pRenderTarget      = nullptr;
                                             SamplerStateParams = { };
                                             VeboxDNDIParams    = { };
+                                            VeboxChromaParams  = { };
                                             pAlphaParams       = nullptr;
                                             // Batch Buffer rendering arguments
                                             BbArgs = { };
@@ -724,6 +732,7 @@ public:
     bool                                bTFF;
     bool                                bTopField;
     bool                                bBeCsc;
+    bool                                bFeCsc;
     bool                                bVeboxBypass;
     bool                                b60fpsDi;
     bool                                bQueryVariance;
@@ -758,7 +767,9 @@ public:
 
     MHW_VEBOX_DNDI_PARAMS               VeboxDNDIParams;
 
-    PVPHAL_ALPHA_PARAMS                 pAlphaParams;
+    MHW_VEBOX_CHROMA_PARAMS             VeboxChromaParams;
+
+    PVPHAL_ALPHA_PARAMS                 pAlphaParams;                      
 
     // Batch Buffer rendering arguments
     VPHAL_ADVPROC_BB_ARGS               BbArgs;
@@ -909,6 +920,8 @@ public:
       PVPHAL_SURFACE                  pInSurface,
       PVPHAL_SURFACE                  pOutSurface);
 
+    void DestorySfcTempSurface();
+
     // External components
     PMHW_VEBOX_INTERFACE            m_pVeboxInterface;                            //!< Pointer to MHW Vebox Structure Interface
     PMHW_SFC_INTERFACE              m_pSfcInterface;                              //!< Pointer to SFC Structure Interface
@@ -928,6 +941,11 @@ public:
     float                           fCscInOffset[3];                            //!< [3x1] Input Offset matrix for CSC
     float                           fCscOutOffset[3];                           //!< [3x1] Output Offset matrix for CSC
 
+    // Front End CSC in VEBOX params
+    float                           *fFeCscCoeff;                                //!< [3x3] Coeff matrix for CSC
+    float                           *fFeCscInOffset;                             //!< [3x1] Input Offset matrix for CSC
+    float                           *fFeCscOutOffset;                            //!< [3x1] Output Offset matrix for CSC
+
     // Dynamic linking filter
     Kdll_FilterEntry                SearchFilter[2];
 
@@ -1038,8 +1056,8 @@ public:
     MOS_GPU_CONTEXT                  RenderGpuContext;                           //!< Render GPU context
 
     VPHAL_SURFACE                    Vebox3DLookUpTables = {};
-    VPHAL_SURFACE                    SfcTempSurface = {};
-    VPHAL_SURFACE                    Sfc2ndTempSurface = {};
+    PVPHAL_SURFACE                   m_sfcTempSurface    = nullptr;
+    PVPHAL_SURFACE                   m_sfc2ndTempSurface = nullptr;
 
     VphalHVSDenoiser                 *m_hvsDenoiser;                             //!< Human Vision System Based Denoiser - Media Kernel to generate DN parameter
     uint8_t                          *m_hvsKernelBinary;                         //!< Human Vision System Based Denoiser - Pointer to HVS kernel Binary
@@ -1781,6 +1799,24 @@ protected:
         int32_t*                            pStatSlice0Offset,
         int32_t*                            pStatSlice1Offset);
 
+    //!
+    //! \brief    Vebox get statistics surface base
+    //! \details  Calculate address of statistics surface address based on the
+    //!           functions which were enabled in the previous call.
+    //! \param    uint8_t* pStat
+    //!           [in] Pointer to Statistics surface
+    //! \param    uint8_t* * pStatSlice0Base
+    //!           [out] Statistics surface Slice 0 base pointer
+    //! \param    uint8_t* * pStatSlice1Base
+    //!           [out] Statistics surface Slice 1 base pointer
+    //! \return   MOS_STATUS
+    //!           Return MOS_STATUS_SUCCESS if successful, otherwise failed
+    //!
+    MOS_STATUS VeboxGetStatisticsSurfaceBase(
+        uint8_t * pStat,
+        uint8_t **pStatSlice0Base,
+        uint8_t **pStatSlice1Base);
+
     //!
     //! \brief    Check if 2 passes CSC are supported on the platform
     //!
