@@ -249,16 +249,33 @@ MOS_STATUS MhwSfcInterfaceG12::AddSfcState(
     {
         cmd.DW19.OutputFrameSurfaceBaseAddressMemoryCompressionEnable = pSfcStateParamsG12->bMMCEnable;
     }
-    cmd.DW19.OutputFrameSurfaceBaseAddressIndexToMemoryObjectControlStateMocsTables = m_outputSurfCtrl.Gen9.Index;
+    cmd.DW19.OutputFrameSurfaceBaseAddressIndexToMemoryObjectControlStateMocsTables = m_outputSurfCtrl.Gen12.Index;
 
     cmd.DW19.OutputFrameSurfaceBaseAddressMemoryCompressionMode = (pSfcStateParamsG12->MMCMode == MOS_MMC_RC) ? 1 : 0;
 
+    switch (pSfcStateParamsG12->tileType)
+    {
+    case MOS_TILE_YF:
+        cmd.DW19.OutputSurfaceTiledMode = TRMODE_TILEYF;
+        break;
+    case MOS_TILE_YS:
+        cmd.DW19.OutputSurfaceTiledMode = TRMODE_TILEYS;
+        break;
+    default:
+        cmd.DW19.OutputSurfaceTiledMode = TRMODE_NONE;
+        break;
+    }
+
     // Set DW22
     cmd.DW22.AvsLineBufferBaseAddressIndexToMemoryObjectControlStateMocsTables
-                                                   = m_avsLineBufferCtrl.Gen9.Index;
+                                                   = m_avsLineBufferCtrl.Gen12.Index;
     // Set DW25
     cmd.DW25.IefLineBufferBaseAddressIndexToMemoryObjectControlStateMocsTables
-                                                   = m_iefLineBufferCtrl.Gen9.Index;
+                                                   = m_iefLineBufferCtrl.Gen12.Index;
+
+    // Set DW28
+    cmd.DW28.SfdLineBufferBaseAddressIndexToMemoryObjectControlStateMocsTables
+                                                   = m_sfdLineBufferCtrl.Gen12.Index;
 
     // Set DW29
     cmd.DW29.OutputSurfaceTileWalk       = (pOutSurface->TileType == MOS_TILE_Y) ?
@@ -316,7 +333,6 @@ MOS_STATUS MhwSfcInterfaceG12::AddSfcState(
         ResourceParams.dwLocationInCmd             = 20;
         ResourceParams.HwCommandType               = MOS_SFC_STATE;
         ResourceParams.bIsWritable                 = true;
-
         MHW_CHK_STATUS_RETURN(pfnAddResourceToCmd(
           pOsInterface,
           pCmdBuffer,
@@ -410,12 +426,18 @@ MOS_STATUS MhwSfcInterfaceG12::AddSfcState(
         ResourceParams.dwLocationInCmd             = 47;
         ResourceParams.HwCommandType               = MOS_SFC_STATE;
         ResourceParams.bIsWritable                 = true;
+        InitMocsParams(ResourceParams, &cmd.DW40.Value, 1, 6);
 
         MHW_CHK_STATUS_RETURN(pfnAddResourceToCmd(
             pOsInterface,
             pCmdBuffer,
             &ResourceParams));
     }
+    //Set DW40, DW43, DW46, DW49
+    cmd.DW40.AvsLineTileBufferBaseAddressIndexToMemoryObjectControlStateMocsTables = m_avsLineBufferCtrl.Gen12.Index;
+    cmd.DW43.IefLineTileBufferBaseAddressIndexToMemoryObjectControlStateMocsTables = m_iefLineBufferCtrl.Gen12.Index;
+    cmd.DW46.SfdLineTileBufferBaseAddressIndexToMemoryObjectControlStateMocsTables = m_sfdLineBufferCtrl.Gen12.Index;
+    cmd.DW49.HistogramBaseAddressMOCSIndex                                         = m_histogramBufferCtrl.Gen12.Index;
 
     MHW_CHK_STATUS_RETURN(Mos_AddCommand(pCmdBuffer, &cmd, cmd.byteSize));
 
@@ -785,6 +807,21 @@ MhwSfcInterfaceG12::MhwSfcInterfaceG12(PMOS_INTERFACE pOsInterface) : MhwSfcInte
     m_iefLineBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
         MOS_MHW_RESOURCE_USAGE_Sfc_IefLineBufferSurface,
         m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
+    m_sfdLineBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
+        MOS_CODEC_RESOURCE_USAGE_SURFACE_UNCACHED,
+        m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
+    m_avsLineTileBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
+        MOS_CODEC_RESOURCE_USAGE_SURFACE_UNCACHED,
+        m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
+    m_iefLineTileBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
+        MOS_CODEC_RESOURCE_USAGE_SURFACE_UNCACHED,
+        m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
+    m_sfdLineTileBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
+        MOS_CODEC_RESOURCE_USAGE_SURFACE_UNCACHED,
+        m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
+    m_histogramBufferCtrl.Value = m_osInterface->pfnCachePolicyGetMemoryObject(
+        MOS_CODEC_RESOURCE_USAGE_SURFACE_UNCACHED,
+        m_osInterface->pfnGetGmmClientContext(m_osInterface)).DwordValue;
 
     m_maxWidth  = MHW_SFC_MAX_WIDTH_G12;
     m_maxHeight = MHW_SFC_MAX_HEIGHT_G12;
