@@ -107,9 +107,9 @@ VAStatus DdiMediaDecode::ParseProcessingBuffer(
             MOS_SecureMemcpy(m_procBuf, sizeof(VAProcPipelineParameterBuffer), procBuf, sizeof(VAProcPipelineParameterBuffer));
         }
         auto decProcessingParams =
-            (PCODECHAL_DECODE_PROCESSING_PARAMS)m_ddiDecodeCtx->DecodeParams.m_procParams;
+            (DecodeProcessingParams *)m_ddiDecodeCtx->DecodeParams.m_procParams;
 
-        auto decProcessingSurface = decProcessingParams->pOutputSurface;
+        auto decProcessingSurface = decProcessingParams->m_outputSurface;
 
         memset(decProcessingSurface, 0, sizeof(MOS_SURFACE));
 
@@ -126,43 +126,67 @@ VAStatus DdiMediaDecode::ParseProcessingBuffer(
         decProcessingSurface->TileType = decProcessingSurface->OsResource.TileType;
         decProcessingSurface->Format   = decProcessingSurface->OsResource.Format;
 
-        decProcessingParams->rcInputSurfaceRegion.X      = procBuf->surface_region->x;
-        decProcessingParams->rcInputSurfaceRegion.Y      = procBuf->surface_region->y;
-        decProcessingParams->rcInputSurfaceRegion.Width  = procBuf->surface_region->width;
-        decProcessingParams->rcInputSurfaceRegion.Height = procBuf->surface_region->height;
+        decProcessingParams->m_inputSurfaceRegion.m_x      = procBuf->surface_region->x;
+        decProcessingParams->m_inputSurfaceRegion.m_y      = procBuf->surface_region->y;
+        decProcessingParams->m_inputSurfaceRegion.m_width  = procBuf->surface_region->width;
+        decProcessingParams->m_inputSurfaceRegion.m_height = procBuf->surface_region->height;
 
-        decProcessingParams->pOutputSurface               = decProcessingSurface;
-        decProcessingParams->rcOutputSurfaceRegion.X      = procBuf->output_region->x;
-        decProcessingParams->rcOutputSurfaceRegion.Y      = procBuf->output_region->y;
-        decProcessingParams->rcOutputSurfaceRegion.Width  = procBuf->output_region->width;
-        decProcessingParams->rcOutputSurfaceRegion.Height = procBuf->output_region->height;
+        decProcessingParams->m_outputSurface                = decProcessingSurface;
+        decProcessingParams->m_outputSurfaceRegion.m_x      = procBuf->output_region->x;
+        decProcessingParams->m_outputSurfaceRegion.m_y      = procBuf->output_region->y;
+        decProcessingParams->m_outputSurfaceRegion.m_width  = procBuf->output_region->width;
+        decProcessingParams->m_outputSurfaceRegion.m_height = procBuf->output_region->height;
+
+        // Interpolation flags
+        // Set the vdbox scaling mode
+        uint32_t uInterpolationflags = 0;
+#if VA_CHECK_VERSION(1, 9, 0)
+        uInterpolationflags = procBuf->filter_flags & VA_FILTER_INTERPOLATION_MASK;
+#endif
+
+        switch (uInterpolationflags)
+        {
+#if VA_CHECK_VERSION(1, 9, 0)
+        case VA_FILTER_INTERPOLATION_NEAREST_NEIGHBOR:
+            decProcessingParams->m_scalingMode = CODECHAL_SCALING_NEAREST;
+            break;
+        case VA_FILTER_INTERPOLATION_BILINEAR:
+            decProcessingParams->m_scalingMode = CODECHAL_SCALING_BILINEAR;
+            break;
+        case VA_FILTER_INTERPOLATION_ADVANCED:
+        case VA_FILTER_INTERPOLATION_DEFAULT:
+#endif
+        default:
+            decProcessingParams->m_scalingMode = CODECHAL_SCALING_AVS;
+           break;
+        }
 
         // Chroma siting
         // Set the vertical chroma siting info
-        uint32_t chromaSitingFlags;
+        uint32_t chromaSitingFlags = 0;
         chromaSitingFlags                       = procBuf->input_color_properties.chroma_sample_location & 0x3;
-        decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_NONE;
-        decProcessingParams->uiRotationState    = 0;
-        decProcessingParams->uiBlendState       = 0;
-        decProcessingParams->uiMirrorState      = 0;
+        decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_NONE;
+        decProcessingParams->m_rotationState    = 0;
+        decProcessingParams->m_blendState       = 0;
+        decProcessingParams->m_mirrorState      = 0;
 
         switch (chromaSitingFlags)
         {
         case VA_CHROMA_SITING_VERTICAL_TOP:
-            decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_VERT_TOP;
+            decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_VERT_TOP;
             break;
         case VA_CHROMA_SITING_VERTICAL_CENTER:
-            decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_VERT_CENTER;
+            decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_VERT_CENTER;
             break;
         case VA_CHROMA_SITING_VERTICAL_BOTTOM:
-            decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_VERT_BOTTOM;
+            decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_VERT_BOTTOM;
             break;
         default:
-            decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_NONE;
+            decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_NONE;
             break;
         }
 
-        if (decProcessingParams->uiChromaSitingType != CODECHAL_CHROMA_SITING_NONE)
+        if (decProcessingParams->m_chromaSitingType != CODECHAL_CHROMA_SITING_NONE)
         {
             // Set the horizontal chroma siting info
             chromaSitingFlags = procBuf->input_color_properties.chroma_sample_location & 0xc;
@@ -170,13 +194,13 @@ VAStatus DdiMediaDecode::ParseProcessingBuffer(
             switch (chromaSitingFlags)
             {
             case VA_CHROMA_SITING_HORIZONTAL_LEFT:
-                decProcessingParams->uiChromaSitingType |= CODECHAL_CHROMA_SITING_HORZ_LEFT;
+                decProcessingParams->m_chromaSitingType |= CODECHAL_CHROMA_SITING_HORZ_LEFT;
                 break;
             case VA_CHROMA_SITING_HORIZONTAL_CENTER:
-                decProcessingParams->uiChromaSitingType |= CODECHAL_CHROMA_SITING_HORZ_CENTER;
+                decProcessingParams->m_chromaSitingType |= CODECHAL_CHROMA_SITING_HORZ_CENTER;
                 break;
             default:
-                decProcessingParams->uiChromaSitingType = CODECHAL_CHROMA_SITING_NONE;
+                decProcessingParams->m_chromaSitingType = CODECHAL_CHROMA_SITING_NONE;
                 break;
             }
         }
@@ -220,8 +244,10 @@ VAStatus DdiMediaDecode::BeginPicture(
     m_streamOutEnabled              = false;
     m_ddiDecodeCtx->DecodeParams.m_numSlices       = 0;
     m_ddiDecodeCtx->DecodeParams.m_dataSize        = 0;
+    m_ddiDecodeCtx->DecodeParams.m_dataOffset      = 0;
     m_ddiDecodeCtx->DecodeParams.m_deblockDataSize = 0;
     m_ddiDecodeCtx->DecodeParams.m_executeCallIndex = 0;
+    m_ddiDecodeCtx->DecodeParams.m_cencBuf         = nullptr;
     m_groupIndex                                   = 0;
 
     // register render targets
@@ -345,8 +371,11 @@ void DdiMediaDecode::DestroyContext(VADriverContextP ctx)
 
     if (codecHal != nullptr)
     {
-        MOS_FreeMemory(codecHal->GetOsInterface()->pOsContext->pPerfData);
-        codecHal->GetOsInterface()->pOsContext->pPerfData = nullptr;
+        if (codecHal->GetOsInterface() && codecHal->GetOsInterface()->pOsContext)
+        {
+            MOS_FreeMemory(codecHal->GetOsInterface()->pOsContext->pPerfData);
+            codecHal->GetOsInterface()->pOsContext->pPerfData = nullptr;
+        }
 
         // destroy codechal
         codecHal->Destroy();
@@ -399,9 +428,9 @@ void DdiMediaDecode::DestroyContext(VADriverContextP ctx)
     if (m_ddiDecodeCtx->DecodeParams.m_procParams != nullptr)
     {
         auto procParams =
-            (PCODECHAL_DECODE_PROCESSING_PARAMS)m_ddiDecodeCtx->DecodeParams.m_procParams;
-        MOS_FreeMemory(procParams->pOutputSurface);
-        procParams->pOutputSurface = nullptr;
+            (DecodeProcessingParams *)m_ddiDecodeCtx->DecodeParams.m_procParams;
+        MOS_FreeMemory(procParams->m_outputSurface);
+        procParams->m_outputSurface = nullptr;
 
         MOS_FreeMemory(m_ddiDecodeCtx->DecodeParams.m_procParams);
         m_ddiDecodeCtx->DecodeParams.m_procParams = nullptr;
@@ -657,7 +686,7 @@ VAStatus DdiMediaDecode::SetDecodeParams()
 
     if (m_ddiDecodeCtx->pCpDdiInterface)
     {
-        DDI_CHK_RET(m_ddiDecodeCtx->pCpDdiInterface->SetDecodeParams(&m_ddiDecodeCtx->DecodeParams),"SetDecodeParams failed!");
+        DDI_CHK_RET(m_ddiDecodeCtx->pCpDdiInterface->SetDecodeParams(m_ddiDecodeCtx, m_codechalSettings),"SetDecodeParams failed!");
     }
 
     Mos_Solo_OverrideBufferSize(m_ddiDecodeCtx->DecodeParams.m_dataSize, m_ddiDecodeCtx->DecodeParams.m_dataBuffer);
@@ -675,50 +704,55 @@ VAStatus DdiMediaDecode::ExtraDownScaling(
     PDDI_MEDIA_CONTEXT mediaCtx = DdiMedia_GetMediaContext(ctx);
     DDI_CHK_NULL(mediaCtx, "nullptr ctx", VA_STATUS_ERROR_INVALID_CONTEXT);
     DDI_CHK_NULL(m_ddiDecodeCtx, "nullptr ctx", VA_STATUS_ERROR_INVALID_CONTEXT);
-#ifdef _APOGEIOS_SUPPORTED
+
+    bool isDecodeDownScalingSupported = false;
     if (m_ddiDecodeCtx->pCodecHal->IsApogeiosEnabled())
     {
-        return MOS_STATUS_SUCCESS; //Add APO logic here.
+        DecodePipelineAdapter *decoder = dynamic_cast<DecodePipelineAdapter *>(m_ddiDecodeCtx->pCodecHal);
+        DDI_CHK_NULL(decoder, "nullptr decoder", VA_STATUS_ERROR_INVALID_PARAMETER);
+        isDecodeDownScalingSupported = decoder->IsDownSamplingSupported();
     }
     else
-#endif
     {
         CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(m_ddiDecodeCtx->pCodecHal);
         DDI_CHK_NULL(decoder, "nullptr decoder", VA_STATUS_ERROR_INVALID_PARAMETER);
-        if(m_ddiDecodeCtx->DecodeParams.m_procParams &&
-            !decoder->IsVdSfcSupported())
+        isDecodeDownScalingSupported = decoder->IsVdSfcSupported();
+    }
+
+    if(m_ddiDecodeCtx->DecodeParams.m_procParams != nullptr &&
+       m_procBuf &&
+       !isDecodeDownScalingSupported)
+    {
+        //check vp context
+        VAContextID vpCtxID = VA_INVALID_ID;
+        if (mediaCtx->pVpCtxHeap != nullptr && mediaCtx->pVpCtxHeap->pHeapBase != nullptr)
         {
-            //check vp context
-            VAContextID vpCtxID = VA_INVALID_ID;
-            if (mediaCtx->pVpCtxHeap != nullptr && mediaCtx->pVpCtxHeap->pHeapBase != nullptr)
-            {
-                //Get VP Context from heap.
-                vpCtxID = (VAContextID)(0 + DDI_MEDIA_VACONTEXTID_OFFSET_VP);
-            }
-            else
-            {
-                //Create VP Context.
-                vaStatus = DdiVp_CreateContext(ctx, 0, 0, 0, 0, 0, 0, &vpCtxID);
-                DDI_CHK_RET(vaStatus, "Create VP Context failed.");
-            }
-
-            uint32_t        ctxType;
-            PDDI_VP_CONTEXT pVpCtx = (PDDI_VP_CONTEXT)DdiMedia_GetContextFromContextID(ctx, vpCtxID, &ctxType);
-            DDI_CHK_NULL(pVpCtx, "nullptr pVpCtx", VA_STATUS_ERROR_INVALID_CONTEXT);
-
-            //Set parameters
-            VAProcPipelineParameterBuffer* pInputPipelineParam = m_procBuf;
-            DDI_CHK_NULL(pInputPipelineParam, "nullptr pInputPipelineParam", VA_STATUS_ERROR_ALLOCATION_FAILED);
-
-            vaStatus = DdiVp_BeginPicture(ctx, vpCtxID, pInputPipelineParam->additional_outputs[0]);
-            DDI_CHK_RET(vaStatus, "VP BeginPicture failed");
-
-            vaStatus = DdiVp_SetProcPipelineParams(ctx, pVpCtx, pInputPipelineParam);
-            DDI_CHK_RET(vaStatus, "VP SetProcPipelineParams failed.");
-
-            vaStatus = DdiVp_EndPicture(ctx, vpCtxID);
-            DDI_CHK_RET(vaStatus, "VP EndPicture failed.");
+            //Get VP Context from heap.
+            vpCtxID = (VAContextID)(0 + DDI_MEDIA_VACONTEXTID_OFFSET_VP);
         }
+        else
+        {
+            //Create VP Context.
+            vaStatus = DdiVp_CreateContext(ctx, 0, 0, 0, 0, 0, 0, &vpCtxID);
+            DDI_CHK_RET(vaStatus, "Create VP Context failed.");
+        }
+
+        uint32_t        ctxType;
+        PDDI_VP_CONTEXT pVpCtx = (PDDI_VP_CONTEXT)DdiMedia_GetContextFromContextID(ctx, vpCtxID, &ctxType);
+        DDI_CHK_NULL(pVpCtx, "nullptr pVpCtx", VA_STATUS_ERROR_INVALID_CONTEXT);
+
+        //Set parameters
+        VAProcPipelineParameterBuffer* pInputPipelineParam = m_procBuf;
+        DDI_CHK_NULL(pInputPipelineParam, "nullptr pInputPipelineParam", VA_STATUS_ERROR_ALLOCATION_FAILED);
+
+        vaStatus = DdiVp_BeginPicture(ctx, vpCtxID, pInputPipelineParam->additional_outputs[0]);
+        DDI_CHK_RET(vaStatus, "VP BeginPicture failed");
+
+        vaStatus = DdiVp_SetProcPipelineParams(ctx, pVpCtx, pInputPipelineParam);
+        DDI_CHK_RET(vaStatus, "VP SetProcPipelineParams failed.");
+
+        vaStatus = DdiVp_EndPicture(ctx, vpCtxID);
+        DDI_CHK_RET(vaStatus, "VP EndPicture failed.");
     }
 #endif
     return MOS_STATUS_SUCCESS;
@@ -779,7 +813,6 @@ VAStatus DdiMediaDecode::InitDummyReference(CodechalDecode& decoder)
     return VA_STATUS_SUCCESS;
 }
 
-#ifdef _APOGEIOS_SUPPORTED
 VAStatus DdiMediaDecode::InitDummyReference(DecodePipelineAdapter& decoder)
 {
     PMOS_SURFACE dummyReference = decoder.GetDummyReference();
@@ -834,7 +867,6 @@ VAStatus DdiMediaDecode::InitDummyReference(DecodePipelineAdapter& decoder)
 
     return VA_STATUS_SUCCESS;
 }
-#endif
 
 VAStatus DdiMediaDecode::EndPicture(
     VADriverContextP ctx,
@@ -864,7 +896,6 @@ VAStatus DdiMediaDecode::EndPicture(
             &m_ddiDecodeCtx->DecodeParams.m_destSurface->OsResource,
             m_ddiDecodeCtx->DecodeParams.m_destSurface);
 
-    #ifdef _APOGEIOS_SUPPORTED
         if(m_ddiDecodeCtx->pCodecHal->IsApogeiosEnabled())
         {
             DecodePipelineAdapter *decoder = dynamic_cast<DecodePipelineAdapter *>(m_ddiDecodeCtx->pCodecHal);
@@ -872,7 +903,6 @@ VAStatus DdiMediaDecode::EndPicture(
             DDI_CHK_RET(InitDummyReference(*decoder), "InitDummyReference failed!");
         }
         else
-    #endif
         {
             CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(m_ddiDecodeCtx->pCodecHal);
             DDI_CHK_NULL(decoder, "Null decoder", VA_STATUS_ERROR_INVALID_PARAMETER);
@@ -1016,6 +1046,14 @@ VAStatus DdiMediaDecode::CreateBuffer(
             buf->pData      = (uint8_t*)MOS_AllocAndZeroMemory(size * numElements);
             buf->format     = Media_Format_CPU;
             break;
+#if VA_CHECK_VERSION(1, 10, 0)
+        case VAContextParameterUpdateBufferType:
+        {
+            buf->pData      = (uint8_t*)MOS_AllocAndZeroMemory(size * numElements);
+            buf->format     = Media_Format_CPU;
+            break;
+        }
+#endif
         default:
             va = m_ddiDecodeCtx->pCpDdiInterface->CreateBuffer(type, buf, size, numElements);
             if (va  == VA_STATUS_ERROR_UNSUPPORTED_BUFFERTYPE)
@@ -1058,13 +1096,10 @@ VAStatus DdiMediaDecode::CreateBuffer(
 
     if( true == buf->bCFlushReq )
     {
-        mos_bo_subdata(buf->bo, buf->uiOffset, size * numElements, data);
-    }
-    else
-    {
-        status = MOS_SecureMemcpy((void *)(buf->pData + buf->uiOffset), size * numElements, data, size * numElements);
-        DDI_CHK_CONDITION((status != MOS_STATUS_SUCCESS), "DDI:Failed to copy buffer data!", VA_STATUS_ERROR_OPERATION_FAILED);
+        mos_bo_wait_rendering(buf->bo);
     }
+    status = MOS_SecureMemcpy((void *)(buf->pData + buf->uiOffset), size * numElements, data, size * numElements);
+    DDI_CHK_CONDITION((status != MOS_STATUS_SUCCESS), "DDI:Failed to copy buffer data!", VA_STATUS_ERROR_OPERATION_FAILED);
     return va;
 
 CleanUpandReturn:
@@ -1133,7 +1168,6 @@ VAStatus DdiMediaDecode::CreateCodecHal(
         return vaStatus;
     }
 
-#ifdef _APOGEIOS_SUPPORTED
     if (codecHal->IsApogeiosEnabled())
     {
         DecodePipelineAdapter *decoder = dynamic_cast<DecodePipelineAdapter *>(codecHal);
@@ -1145,7 +1179,6 @@ VAStatus DdiMediaDecode::CreateCodecHal(
         }
     }
     else
-#endif
     {
         CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(codecHal);
         if (nullptr == decoder)
@@ -1244,7 +1277,6 @@ void DdiMediaDecode::GetDummyReferenceFromDPB(
 
     if (i < DDI_MEDIA_MAX_SURFACE_NUMBER_CONTEXT)
     {
-    #ifdef _APOGEIOS_SUPPORTED
         if (decodeCtx->pCodecHal->IsApogeiosEnabled())
         {
             DecodePipelineAdapter *decoder = dynamic_cast<DecodePipelineAdapter *>(decodeCtx->pCodecHal);
@@ -1256,7 +1288,6 @@ void DdiMediaDecode::GetDummyReferenceFromDPB(
             decoder->GetDummyReference()->OsResource = dummyReference.OsResource;
         }
         else
-    #endif
         {
             CodechalDecode *decoder = dynamic_cast<CodechalDecode *>(decodeCtx->pCodecHal);
             if (decoder == nullptr)
@@ -1280,32 +1311,36 @@ void DdiMediaDecode::ReportDecodeMode(
         case CODECHAL_DECODE_MODE_MPEG2IDCT:
         case CODECHAL_DECODE_MODE_MPEG2VLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_MPEG2_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_VC1IT:
         case CODECHAL_DECODE_MODE_VC1VLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_VC1_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_AVCVLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_AVC_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_JPEG:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_JPEG_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_VP8VLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_VP8_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_HEVCVLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_HEVC_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         case CODECHAL_DECODE_MODE_VP9VLD:
             userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_VP9_MODE_ID;
-            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1);
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
+            break;
+        case CODECHAL_DECODE_MODE_AV1VLD:
+            userFeatureWriteData.ValueID = __MEDIA_USER_FEATURE_VALUE_DECODE_AV1_MODE_ID;
+            MOS_UserFeature_WriteValues_ID(nullptr, &userFeatureWriteData, 1, nullptr);
             break;
         default:
             break;
