@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2017-2020, Intel Corporation
+* Copyright (c) 2017-2021, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -315,6 +315,43 @@ public:
             VASurfaceAttrib *attribList,
             uint32_t *numAttribs);
 
+    //!
+    //! \brief    Query display attributes
+    //!
+    //! \param    [in, out] attribList
+    //!           it returns the supported display attributes
+    //!
+    //!
+    //! \param    [in, out] numAttribs
+    //!           it returns the actual number of supported attributes
+    //!
+    //! \return   VAStatus
+    //!           VA_STATUS_SUCCESS if success
+    //!           VA_STATUS_ERROR_MAX_NUM_EXCEEDED if size of attribList is too small
+    //!
+    virtual VAStatus QueryDisplayAttributes(
+            VADisplayAttribute *attribList,
+            int32_t *numAttribs);
+
+    //!
+    //! \brief    Get display attributes
+    //!           returns the current attributes values in "attribList"
+    //!
+    //! \param    [in, out] attribList
+    //!           the attrib type should be filled.
+    //!           returns the supported display attributes
+    //!
+    //! \param    [in] numAttribs
+    //!           the number of supported attributes
+    //!
+    //! \return   VAStatus
+    //!           VA_STATUS_SUCCESS if success
+    //!           VA_STATUS_ERROR_MAX_NUM_EXCEEDED if size of attribList is too small
+    //!
+    virtual VAStatus GetDisplayAttributes(
+            VADisplayAttribute *attribList,
+            int32_t numAttribs);
+  
     //!
     //! \brief    Check if the resolution is valid for a given decode codec mode
     //!
@@ -695,6 +732,18 @@ public:
         return VA_STATUS_SUCCESS;
     }
 
+    //! \brief Get surface drm modifier
+    //!
+    //! \param    [in] mediaSurface
+    //!           Pointer to the media surface
+    //! \param    [out] modifier
+    //!           reference of the modifier
+    //!
+    //! \return   VAStatus
+    //!           VA_STATUS_SUCCESS if success
+    //!
+    virtual VAStatus GetSurfaceModifier(DDI_MEDIA_SURFACE* mediaSurface, uint64_t &modifier);
+
 protected:
     //!
     //! \class    ProfileEntrypoint
@@ -746,7 +795,8 @@ protected:
     {
         videoEncode, //!< Video encode
         videoDecode, //!< Video decode
-        videoProcess //!< Video processing
+        videoProcess,//!< Video processing
+        videoProtect //!< Video protection
     };
 
     enum EncodeFormat
@@ -770,15 +820,20 @@ protected:
         uint32_t        colorFormat;
     };
 
+#if VA_CHECK_VERSION(1, 10, 0)
+    static const uint32_t m_numEncRcMode = 10;
+#else
+    static const uint32_t m_numEncRcMode = 9;
+#endif
     static const uint16_t m_maxProfiles = 17; //!< Maximum number of supported profiles
     static const uint16_t m_maxProfileEntries = 64; //!< Maximum number of supported profile & entrypoint combinations
-    static const uint32_t m_numVpSurfaceAttr = 18; //!< Number of VP surface attributes
+    static const uint32_t m_numVpSurfaceAttr = 20; //!< Number of VP surface attributes
     static const uint32_t m_numJpegSurfaceAttr = 7; //!< Number of JPEG surface attributes
     static const uint32_t m_numJpegEncSurfaceAttr = 4; //!< Number of JPEG encode surface attributes
     static const uint16_t m_maxEntrypoints = 7; //!<  Maximum number of supported entrypoints
     static const uint32_t m_decSliceMode[2]; //!< Store 2 decode slices modes
     static const uint32_t m_decProcessMode[2]; //!< Store 2 decode process modes
-    static const uint32_t m_encRcMode[9]; //!< Store 9 encode rate control modes
+    static const uint32_t m_encRcMode[m_numEncRcMode]; //!< Store encode rate control modes
     static const uint32_t m_vpSurfaceAttr[m_numVpSurfaceAttr]; //!< Store the VP surface attributes
     static const uint32_t m_jpegSurfaceAttr[m_numJpegSurfaceAttr]; //!< Store the JPEG surface attributes
     static const uint32_t m_jpegEncSurfaceAttr[m_numJpegEncSurfaceAttr]; //!< Store the JPEG encode surface attributes
@@ -1117,7 +1172,7 @@ protected:
     //!
     //! \brief    Initialize AVC Low-power encode profiles, entrypoints and attributes
     //!
-    VAStatus LoadAvcEncLpProfileEntrypoints();
+    virtual VAStatus LoadAvcEncLpProfileEntrypoints();
 
     //!
     //! \brief    Initialize MPEG2 decode profiles, entrypoints and attributes
@@ -1127,7 +1182,7 @@ protected:
     //!
     //! \brief    Initialize MPEG2 encode profiles, entrypoints and attributes
     //!
-    VAStatus LoadMpeg2EncProfileEntrypoints();
+    virtual VAStatus LoadMpeg2EncProfileEntrypoints();
 
     //!
     //! \brief    Initialize JPEG decode profiles, entrypoints and attributes
@@ -1137,7 +1192,7 @@ protected:
     //!
     //! \brief    Initialize JPEG encode profiles, entrypoints and attributes
     //!
-    VAStatus LoadJpegEncProfileEntrypoints();
+    virtual VAStatus LoadJpegEncProfileEntrypoints();
 
     //!
     //! \brief    Initialize VC1 decode profiles, entrypoints and attributes
@@ -1350,7 +1405,7 @@ protected:
     //! \return   VAStatus
     //!           VA_STATUS_SUCCESS if success
     //!
-    VAStatus CheckEncRTFormat(
+    virtual VAStatus CheckEncRTFormat(
             VAProfile profile,
             VAEntrypoint entrypoint,
             VAConfigAttrib* attrib);
@@ -1387,5 +1442,6 @@ protected:
     //!           VA_STATUS_SUCCESS if success
     //!
     VAStatus GetGeneralConfigAttrib(VAConfigAttrib* attrib);
+
 };
 #endif
